//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file calculates acceleration response spectrum of an input ground motion which is in "g".
// This file reads the input acceleration time history, use Newmark-beta method to calculate spectral acceleration for different system periods, and finally writes the results in a user-defined file.
//===========================================================================================================================

#include "Spectrum.hpp"

//Default constructor
Spectrum::Spectrum() {
    // Do nothing
}

//Destructor
Spectrum::~Spectrum() {
    // Do nothing
}

//Set values
void Spectrum::setValues (double dt , double zeta, double gravity, string motionDir, string outputDir) {
    this-> dt =dt;
    this-> zeta =zeta;
    this-> gravity =gravity;
    this-> motionDir =motionDir;
    this-> outputDir =outputDir;
}


// This function reads the input file
void Spectrum::readfiles() {

    ifstream inputData;
    inputData.open (motionDir);

    numData=0;
    double col1;
    double col2;

    if ( inputData.fail()) {
        cerr << "Error in opening the input file" << endl;
        //exit(1);
    } else {
        while (! inputData.eof() ) {
            inputData >> col1 >> col2;
            Time.push_back(col1);
            ddug.push_back(col2*gravity);
            numData++;
        }
        inputData.close();
    }
}

//This function calculates Acceleration Response Spectrum
void Spectrum::computeARS() {

    readfiles();

    for (int k = 0; k < maxT/dT; k++) {

        Period.push_back(dT*(k+1));
        omega =2.0*3.1416/Period[k];

        double maxvalue=0.0;

        ddu.clear();
        du.clear();
        u.clear();
        ddu_total.clear();

        ddu.push_back(0.0);
        du.push_back(0.0);
        u.push_back(0.0);
        ddu_total.push_back(0.0);

        for (int i =0; i< numData; i++) {

            double term1 = -ddug[i+1] - (pow (omega,2.0)*(u[i]+dt*du[i]+(1-2*beta)/2.0 * pow (dt,2.0) * ddu[i])) -(2.*zeta/100.0*omega*(du[i] + (1-gamma)*dt * ddu[i]));
            double term2 = 1.0 + pow (omega,2.0)*beta*pow (dt,2.0) + 2.*zeta/100.0*omega*gamma*dt;

            ddu.push_back(term1/term2);                 //relative acceleration
            ddu_total.push_back(ddu[i+1] + ddug[i+1]);  // absolute acceleration

            du.push_back(du[i] + (1-gamma)*dt * ddu[i]+ gamma*dt*ddu[i+1]);
            u.push_back(u[i]+dt*du[i]+(1-2*beta)/2.0 * pow (dt,2.0) * ddu[i]+ beta*pow (dt,2.0)*ddu[i+1]);

            if ( abs(ddu_total[i+1]) > maxvalue ) {
                maxvalue=abs(ddu_total[i+1]);
            }

        }

        Sa.push_back(maxvalue/gravity);

    }
    writefiles();
}

// This function writes Period-Spectral Acceleration values in a user-defined output file
void Spectrum::writefiles() {

    ofstream outputData;
    outputData.open (outputDir);

    int numSpect=(int)(maxT/dT);

	outputData << "Period(sec)" << " " << "Sa(g)" << endl;
    for (int i=0; i < numSpect; i++) {
        outputData << Period[i] << " " << Sa[i] << endl;
    }

    outputData.close();

}


